==> sexpr-0.0.0.tar.gz: lib/sexpr.dart <==
import 'package:code_buffer/code_buffer.dart';

SExpr _toSExpr(x) {
  if (x is SExpr) return x;
  if (x is SExprBuilder) return x.toSExpr();
  return atom(x);
}

/// Shorthand for [ListSExpr].
///
/// [callee] and [arguments] can be any Dart value, or a [SExpr] or [SExprBuilder].
ListSExpr list(callee, Iterable arguments) =>
    ListSExpr(_toSExpr(callee), arguments.map(_toSExpr));

/// Shorthand for [AtomSExpr].
AtomSExpr atom(text) => AtomSExpr(text is String ? text : text.toString());

/// Builds an [SExpr] somehow.
abstract class SExprBuilder {
  /// Creates a new [SExpr] instance representing `this` object.
  SExpr toSExpr();
}

/// An arbitrary s-expression. Can be an `atom` or `list`.
abstract class SExpr {
  /// Generates indentation-aware text.
  void generate(CodeBuffer buffer);

  @override
  String toString() {
    var buffer = CodeBuffer();
    generate(buffer);
    return buffer.toString();
  }
}

/// An [SExpr] that exists as a standalone expression, and is not invoked as a function.
class AtomSExpr extends SExpr {
  /// The textual value of this atom. It will be outputted as-is.
  final String text;

  AtomSExpr(this.text);

  @override
  void generate(CodeBuffer buffer) {
    buffer.write(text);
  }
}

/// An [SExpr] that consists of a single [callee], and a number of [arguments].
class ListSExpr extends SExpr {
  /// The object that is being invoked.
  final SExpr callee;

  /// All arguments being passed to this invocation.
  final List<SExpr> arguments = [];

  ListSExpr(this.callee, [Iterable<SExpr> arguments = const []]) {
    this.arguments.addAll(arguments ?? []);
  }

  @override
  void generate(CodeBuffer buffer) {
    buffer.write('(');
    callee.generate(buffer);

    // Generate all the children.
    // Try to fit them all onto one line.
    var args = arguments.map((a) {
      var b = CodeBuffer();
      a.generate(b);
      return b;
    });

    if (args.isEmpty) {
      buffer.write(')');
      return;
    }

    // If any argument spans more than one line, we obviously
    // can't fit them all onto one line.
    if (!args.any((b) => b.lines.length > 1)) {
      // Sum up their lengths.
      var totalLength = args.map((b) => b.length).reduce((a, b) => a + b);
      totalLength += args.length; // Account for spaces placed between
      totalLength +=
          buffer.lastLine.span.length; // Account for text on the current line

      // Limit the max length to 80 characters.
      if (totalLength <= 80) {
        for (var arg in args) {
          buffer.write(' ');
          buffer.write(arg.lines[0].text);
        }

        buffer.write(')');
        return;
      }
    }

    // Otherwise, just print them all on new lines.
    for (var arg in arguments) {
      var b = CodeBuffer();
      arg.generate(b);
      buffer.writeln();
      buffer.indent();
      b.copyInto(buffer);
      buffer.outdent();
    }

    buffer.write(')');
  }
}

==> sexpr-1.0.0.tar.gz: lib/sexpr.dart <==
import 'package:code_buffer/code_buffer.dart';

SExpr _toSExpr(x) {
  if (x is SExpr) return x;
  if (x is SExprBuilder) return x.toSExpr();
  return atom(x);
}

/// Shorthand for [ListSExpr].
///
/// [callee] and [arguments] can be any Dart value, or a [SExpr] or [SExprBuilder].
ListSExpr list(callee, Iterable arguments) =>
    new ListSExpr(_toSExpr(callee), arguments.map(_toSExpr));

/// Shorthand for [AtomSExpr].
AtomSExpr atom(text) => new AtomSExpr(text is String ? text : text.toString());

/// Builds an [SExpr] somehow.
abstract class SExprBuilder {
  /// Creates a new [SExpr] instance representing `this` object.
  SExpr toSExpr();
}

/// An arbitrary s-expression. Can be an `atom` or `list`.
abstract class SExpr {
  /// Generates indentation-aware text.
  void generate(CodeBuffer buffer);

  @override
  String toString() {
    var buffer = new CodeBuffer();
    generate(buffer);
    return buffer.toString();
  }
}

/// An [SExpr] that exists as a standalone expression, and is not invoked as a function.
class AtomSExpr extends SExpr {
  /// The textual value of this atom. It will be outputted as-is.
  final String text;

  AtomSExpr(this.text);

  @override
  void generate(CodeBuffer buffer) {
    buffer.write(text);
  }
}

/// An [SExpr] that consists of a single [callee], and a number of [arguments].
class ListSExpr extends SExpr {
  /// The object that is being invoked.
  final SExpr callee;

  /// All arguments being passed to this invocation.
  final List<SExpr> arguments = [];

  ListSExpr(this.callee, [Iterable<SExpr> arguments = const []]) {
    this.arguments.addAll(arguments ?? []);
  }

  @override
  void generate(CodeBuffer buffer) {
    buffer.write('(');
    callee.generate(buffer);

    // Generate all the children.
    // Try to fit them all onto one line.
    var args = arguments.map((a) {
      var b = new CodeBuffer();
      a.generate(b);
      return b;
    });

    if (args.isEmpty) {
      buffer.write(')');
      return;
    }

    // If any argument spans more than one line, we obviously
    // can't fit them all onto one line.
    if (!args.any((b) => b.lines.length > 1)) {
      // Sum up their lengths.
      var totalLength = args.map((b) => b.length).reduce((a, b) => a + b);
      totalLength += args.length; // Account for spaces placed between
      totalLength +=
          buffer.lastLine.span.length; // Account for text on the current line

      // Limit the max length to 80 characters.
      if (totalLength <= 80) {
        for (var arg in args) {
          buffer.write(' ');
          buffer.write(arg.lines[0].text);
        }

        buffer.write(')');
        return;
      }
    }

    // Otherwise, just print them all on new lines.
    for (var arg in arguments) {
      var b = new CodeBuffer();
      arg.generate(b);
      buffer.writeln();
      buffer.indent();
      b.copyInto(buffer);
      buffer.outdent();
    }

    buffer.write(')');
  }
}
